#!/bin/sh
PATH=/sbin:/usr/sbin:/bin:/usr/bin:$PATH
export PATH

IMSS_HOME=/opt/trend/imss
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/opt/trend/imss/cdt
export IMSS_HOME
export LD_LIBRARY_PATH

displayDates()
{
    clear
    echo "The Case Diagnostic Tool will now collect basic system"
    echo "and Trend Micro IMSA information from your system. "
    echo "------------------------------------------------------"
    echo "Select the dates for which to collect log information:"
    echo "1) Collect Information Today Only"
    echo "2) Collect Information Last Two Days"
    echo "3) Collect Information Last Three Days"
    echo "------------------------------------------------------"
    printf "Select an option: (1-3) [1]:"
    read DAYS
    until [ "$DAYS" == "" ] || [ "$DAYS" == "1" ] || [ "$DAYS" == "2" ] || [ "$DAYS" == "3" ]; do
        printf "Please input a valid option: (1-3) [1]:"
        read DAYS
    done

    if [ "$DAYS" == "" ]; then
        DAYS=1
    fi
}

makeIni()
{
    if test ! -f $IMSS_HOME/cdt/cdt.ini; then
        echo "Can not find $IMSS_HOME/cdt/cdt.ini"
        exit -1
    fi

    BEGINDATE=`date +%Y%m%d -d"$1 days ago"`
    ENDDATE=`date +%Y%m%d`

    sed -i -e "s/^LogFile_Begin.*/LogFile_Begin=$BEGINDATE/" -e "s/^LogFile_End.*/LogFile_End=$ENDDATE/"  $IMSS_HOME/cdt/cdt.ini
}

makePrepare()
{
    mkdir -p /var/app_data/imss/cdt
    rm -rf /var/app_data/imss/cdt/*
    rm -rf $IMSS_HOME/cdt/CDT_DebugLog.log

    echo "==============================Running Processes=========================" > /etc/conf/ps_info.txt 2>&1
    ps aux >> /etc/conf/ps_info.txt 2>&1
    echo "==============================Running Threads===========================" >> /etc/conf/ps_info.txt 2>&1
    ps -eLf >> /etc/conf/ps_info.txt 2>&1
    
    if test -f /opt/trend/imss/patch/patchlog.txt ; then
        cp -f /opt/trend/imss/patch/patchlog.txt /var/log/ 2>/dev/null
    fi
}

makeDumps()
{
    CORES=`ls -1 /var/coredumps/core* 2>/dev/null`

    GDBDIR=/var/coredumps/gdb
    
    rm -rf $GDBDIR
    
    mkdir -p $GDBDIR
    mkdir -p $GDBDIR/dump
    mkdir -p $GDBDIR/script
    
    ls -l /var/coredumps/core_* > $GDBDIR/dump/timestamp_core.txt 2>/dev/null
    echo "info threads" 2>/dev/null > $GDBDIR/script/cmds
    
    for CF in $CORES ; do
        COMM=`echo $CF | awk -F"_" '{print $2}' 2>/dev/null`
        PID=`echo $CF | awk -F"_" '{print $3}' 2>/dev/null`
        ARGS=`ps -o command= -C $COMM 2>/dev/null`
        EXE=`echo $ARGS | head -1 | awk -F" " '{print $1}' 2>/dev/null`
        
        if test "$EXE" != "" ; then
            THREADS=`gdb $EXE $CF -q --batch -x $GDBDIR/script/cmds 2>/dev/null | grep process -c`
            SCRIPT=$GDBDIR/script/core_"$COMM"_$PID.cmd
            touch $SCRIPT
            INDEX=0
            until [ $INDEX -ge $THREADS ] ; do
                INDEX=`expr $INDEX + 1`
                printf "thread $INDEX\nwhere\ninfo register\ndisass\nx/1000 \$esp\n" >> $SCRIPT
            done
    
            gdb $EXE $CF -q --batch 2>/dev/null -x $SCRIPT > $GDBDIR/dump/core_"$COMM"_$PID.gdb
        fi
    done
}

makeDatabase()
{
    DB_EXE_HOME=/opt/trend/imss/PostgreSQL/bin
    DB_EXE_IMSS="$DB_EXE_HOME/psql -q -d imss -U sa -c "
    DB_EXE_IMSSEUQ="$DB_EXE_HOME/psql -q -d imsseuq -U sa -c "
    
    DATABASE_INFO="select oid from pg_database"
    CLASS_INFO="select oid, * from pg_class"
    TABLE_INFO="select * from pg_tables where schemaname='public'"
    LOCK_INFO="select * from pg_locks"
    
    TMP_FILE=/tmp/.asmi
    
    OUT_FILE=/var/imss/pgdata/pg_info
    
    if test -f $OUT_FILE ; then
        rm -f $OUT_FILE
    fi
    
    #if is_master.sh, collect the following information
    
    /opt/trend/imss/script/is_master.sh >/dev/null
    
    if [ $? -eq 1 ] ; then
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imss database                      #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    IMSSOID=`/opt/trend/imss/PostgreSQL/bin/psql -U imss -t -c "select oid from pg_database where datname='imss'"`
    # get imss size
    du -sh `printf /var/imss/pgdata/base/%d $IMSSOID` >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imss pg_class                      #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSS"$CLASS_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imss pg_tables                     #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSS"$TABLE_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imss pg_locks                      #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSS"$LOCK_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#               export settings in imss db               #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    /opt/trend/imss/cdt/imssdump.sh /opt/trend/imss 0 > /opt/trend/imss/cdt/dbdump.log 2>&1
    cat /opt/trend/imss/cdt/dbdump.log >> $OUT_FILE 2>/dev/null

    fi
    
    #if is_euq_enable.sh, collect the following information
    /opt/trend/imss/script/is_euq_enable.sh >/dev/null
    
    if [ $? -eq 1 ] ; then
           
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imsseuq database                   #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    IMSSEUQOID=`/opt/trend/imss/PostgreSQL/bin/psql -U imss -t -c "select oid from pg_database where datname='imsseuq'"`
    
    # get imsseuq size
    du -sh `printf /var/imss/pgdata/base/%d $IMSSEUQOID` >> $OUT_FILE 2>/dev/null
    
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imsseuq pg_class                   #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSSEUQ"$CLASS_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imsseuq pg_tables                  #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSSEUQ"$TABLE_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    echo "##########################################################" >> $OUT_FILE
    echo "#                     imsseuq pg_locks                   #" >> $OUT_FILE
    echo "##########################################################" >> $OUT_FILE
    $DB_EXE_IMSSEUQ"$LOCK_INFO" -o $TMP_FILE 2>/dev/null
    cat $TMP_FILE >> $OUT_FILE 2>/dev/null
    
    fi
    
    # remove tmp file
    
    if test -f $TMP_FILE ; then
        rm -f $TMP_FILE
    fi
}

# main

# check root previlige
if [ `echo $UID` != '0' ]; then
    echo "Must be root to run this script."
    exit -1;
fi

if [ "$1" == "" ]; then
    displayDates
else
    DAYS=$1
fi

if [ $DAYS != "1" ] && [ $DAYS != "2" ] && [ $DAYS != "3" ]; then
    DAYS=1
fi

makeIni $DAYS

#echo "Now starting to collect diagnostic information."

makePrepare

makeDumps

makeDatabase

SAVEFOLDER=`$IMSS_HOME/cdt/cdt | egrep "You can find the diagnostic data" | awk '{ print $8 }'`
ZIPFILE=`echo $SAVEFOLDER | awk -F"/" '{ print "/"$2"/"$3"/"$4"/"$5"/CDT-"$6".zip" }'`
CDTTMP=`echo $SAVEFOLDER | awk -F"/" '{ print "/var/app_data/imss/cdt/"$6 }'`

# remove the cdt temp folder
rm -rf $CDTTMP 2>/dev/null
rm -rf /opt/trend/imss/cdt/dbdump.log 2>/dev/null
rm -rf /opt/trend/imss/cdt/dbdump/admindb1/* 2>/dev/null

#echo $ZIPFILE

if test ! -f $ZIPFILE; then
    echo "Can not find $ZIPFILE"
    exit -1;
fi

#echo "Diagnostic information successfully collected."

printf $ZIPFILE

exit 0
