CREATE OR REPLACE FUNCTION Update_dtas_mail_table () RETURNS integer AS '
DECLARE
    v_szExecuteCmd text;
    v_cnt  int;
BEGIN
    
    select into v_cnt count(*) From pg_attribute 
        where attrelid in(
            select oid from pg_class where relname = ''tb_dtas_mail'') 
            and attname = ''original_msg_id'';

    if (v_cnt > 0) then
        
        /* remove column original_msg_id */
        v_szExecuteCmd := ''alter table tb_dtas_mail drop column original_msg_id;'';
        EXECUTE v_szExecuteCmd;
        
    end if;

    select into v_cnt count(indexrelname) from pg_stat_user_indexes
        where schemaname = ''public'' and indexrelname = ''idx_msg_id'';
    
    if (v_cnt > 0) then
        
        /* add index idx_msg_id*/
        v_szExecuteCmd := ''drop index idx_msg_id;'';
        EXECUTE v_szExecuteCmd;
        
    end if;
        
    /* primary key recreate, remove duplicate rows first */
    select into v_cnt count(indexrelname) from pg_stat_user_indexes
        where schemaname = ''public'' and indexrelname = ''tb_dtas_mail_pkey'';
    
    if (v_cnt = 0) then
    
        /* remove duplicate rows */
        
        v_szExecuteCmd := ''delete from tb_dtas_mail where event_idx <> (select max(b.event_idx) from tb_dtas_mail b where tb_dtas_mail.msg_id = b.msg_id);'';
        EXECUTE v_szExecuteCmd;
                
        /* primary key recreate */
        v_szExecuteCmd := ''ALTER TABLE tb_dtas_mail ADD CONSTRAINT tb_dtas_mail_pkey PRIMARY KEY (msg_id);'';
        EXECUTE v_szExecuteCmd;
        
    end if;
    
    /* reindex */
    v_szExecuteCmd := ''reindex table tb_dtas_mail;'';
    EXECUTE v_szExecuteCmd;
          
    return 0;
    
END;
' LANGUAGE plpgsql;

select Update_dtas_mail_table();
drop function Update_dtas_mail_table();

CREATE OR REPLACE FUNCTION CreateLogImportFunc (int, int) RETURNS int AS $createBody$
DECLARE
    nFirstHour     ALIAS FOR $1;
    nCreateNum     ALIAS FOR $2;
    nTmp           int;
    nHourSeq       int;
    szCreateTableCmd text;
BEGIN
    nTmp := 0;
    while nCreateNum > nTmp loop
    nHourSeq := nFirstHour + nTmp;
    szCreateTableCmd := 'CREATE OR REPLACE FUNCTION AddPolicyEventFromLog_' || nHourSeq || $AddPolicyEventFromLog1$ (
    char(36), int, varchar(321), varchar(2048), varchar(256), smallint, float,
    varchar(256), timestamptz, char(14), varchar(256), char(16), real, int, smallint, varchar(1024),
    varchar(80), varchar(1024),varchar(80), int, timestamptz, int, timestamptz, varchar(1024), bit, int, int, int, varchar(1024), varchar(256), varchar(1024), int)
RETURNS integer AS $AddPolicyEventFromLog_body$
DECLARE
    v_msg_id              ALIAS FOR $1;
    v_scanner_id          ALIAS FOR $2;
    v_smtp_sender         ALIAS FOR $3;
    v_smtp_rcpt           ALIAS FOR $4;
    v_msg_subject         ALIAS FOR $5;
    v_msg_route           ALIAS FOR $6;
    v_msg_size            ALIAS FOR $7;
    v_rule_name           ALIAS FOR $8;
    v_time_date           ALIAS FOR $9;
    v_filter_types        ALIAS FOR $10;
    v_filter_description  ALIAS FOR $11;
    v_msg_actions         ALIAS FOR $12;
    v_spam_score          ALIAS FOR $13;
    v_spam_category_id    ALIAS FOR $14;
    v_spam_sensitive      ALIAS FOR $15;
    v_attach_name         ALIAS FOR $16;
    v_attach_type         ALIAS FOR $17;
    v_virus_name          ALIAS FOR $18;
    v_virus_type          ALIAS FOR $19;
    v_qa_id               ALIAS FOR $20;
    v_qa_live_ttl         ALIAS FOR $21;
    v_aa_id               ALIAS FOR $22;
    v_aa_live_ttl         ALIAS FOR $23;
    v_org_msg_id          ALIAS FOR $24;
    v_multi_virus         ALIAS FOR $25;
    v_ref_num             ALIAS FOR $26;
    v_wrs_threshold       ALIAS FOR $27;
    v_wrs_score           ALIAS FOR $28;
    v_reason              ALIAS FOR $29;
    v_dkim_author         ALIAS FOR $30;
    v_dkim_reason         ALIAS FOR $31;
    v_dda_type            ALIAS FOR $32;
    v_filter_type_virus         bit;
    v_filter_type_spam          bit;
    v_filter_type_attachment    bit;
    v_filter_type_content       bit;
    v_filter_type_size          bit;
    v_filter_type_other         bit;
    v_filter_type_malformed     bit;
    v_filter_type_unscanned     bit;
    v_filter_type_phishing      bit;
    v_filter_type_spyware       bit;
    v_filter_type_wrs           bit;
    v_filter_type_dkim          bit;
    v_filter_type_preflt        bit;
    v_filter_type_compliance    bit;
    v_act_deleted               bit;
    v_act_quarantined           bit;
    v_act_delivered             bit;
    v_act_archived              bit;
    v_act_rcptchanged           bit;
    v_act_stamped               bit;
    v_act_attachments_deleted   bit;
    v_act_subjects_tagged       bit;
    v_act_delivery_postponed    bit;
    v_act_notifications_sent    bit;
    v_act_cleaned               bit;
    v_act_bcc                   bit;
    v_act_handoff               bit;
    v_act_insert_x_header_tag   bit;
    v_act_decryption            bit;
    v_act_encryption            bit;
    v_act_reencryption          bit;
    v_policy_event_id           int4;
    v_len                       int;
    v_pos                       int;
    v_str                       varchar(2048);
    v_suspicious                int;
    v_dda_status                int;
BEGIN  
    /*parse to get filter type*/
    v_filter_type_virus      := substring(v_filter_types from 1 for 1);
    v_filter_type_spam       := substring(v_filter_types from 2 for 1);
    v_filter_type_attachment := substring(v_filter_types from 3 for 1);
    v_filter_type_content    := substring(v_filter_types from 4 for 1);
    v_filter_type_size       := substring(v_filter_types from 5 for 1);
    v_filter_type_other      := substring(v_filter_types from 6 for 1);
    v_filter_type_malformed  := substring(v_filter_types from 7 for 1);
    v_filter_type_unscanned  := substring(v_filter_types from 8 for 1);
    v_filter_type_phishing   := substring(v_filter_types from 9 for 1);
    v_filter_type_wrs        := substring(v_filter_types from 10 for 1);
    v_filter_type_dkim       := substring(v_filter_types from 11 for 1);
    v_filter_type_compliance := substring(v_filter_types from 13 for 1);
    v_filter_type_spyware    := substring(v_filter_types from 14 for 1);
    v_filter_type_preflt     := substring(v_filter_types from 15 for 1);
    /*parse to get action */
    v_act_deleted            := substring(v_msg_actions from 1 for 1);
    v_act_quarantined        := substring(v_msg_actions from 2 for 1);
    v_act_delivered          := substring(v_msg_actions from 3 for 1);
    v_act_archived           := substring(v_msg_actions from 4 for 1);
    v_act_handoff            := substring(v_msg_actions from 5 for 1); 
    v_act_rcptchanged        := substring(v_msg_actions from 6 for 1);
    v_act_stamped            := substring(v_msg_actions from 7 for 1);
    v_act_attachments_deleted:= substring(v_msg_actions from 8 for 1);
    v_act_subjects_tagged    := substring(v_msg_actions from 9 for 1);
    v_act_delivery_postponed := substring(v_msg_actions from 10 for 1);
    v_act_notifications_sent := substring(v_msg_actions from 11 for 1);
    v_act_cleaned            := substring(v_msg_actions from 12 for 1);
    v_act_bcc                := substring(v_msg_actions from 13 for 1);
    v_act_insert_x_header_tag := substring(v_msg_actions from 14 for 1);
    v_act_decryption         := substring(v_msg_actions from 15 for 1);
    v_act_encryption         := substring(v_msg_actions from 16 for 1);
    v_act_reencryption       := substring(v_msg_actions from 17 for 1);
    v_policy_event_id        := nextval('tb_policy_event_$AddPolicyEventFromLog1$ || nHourSeq || $AddPolicyEventFromLog2$_policy_event_id_seq');
    /*suspicious, suspicious + virus*/
    if v_dda_type > 0 then
        v_suspicious := 1;
        if v_dda_type < 10 then
            v_dda_status := 1;
        else
            v_dda_status := 0;
        end if;
    else
        v_suspicious := 0;
        v_dda_status := 0;
    end if;

    /*1.add a row into tb_policy_event, msg_id&scanner_id may be duplicate*/
    insert into tb_policy_event_$AddPolicyEventFromLog2$ || nHourSeq || $AddPolicyEventFromLog31$ values (v_policy_event_id,
        v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size, v_rule_name, v_time_date, 
        v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment, 
        v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed, 
        v_filter_type_unscanned, v_filter_type_phishing, v_filter_description, v_act_deleted, v_act_quarantined,
        v_act_delivered, v_act_archived, v_act_rcptchanged, v_act_stamped, v_act_attachments_deleted, 
        v_act_subjects_tagged, v_act_delivery_postponed, v_act_notifications_sent, v_act_cleaned, v_act_bcc, 
        v_act_handoff, v_spam_score, v_spam_category_id, v_spam_sensitive, v_attach_name, v_attach_type,    
        v_virus_name, v_virus_type, v_org_msg_id, v_filter_type_wrs, v_wrs_threshold, v_wrs_score, v_reason,
        v_filter_type_dkim, v_dkim_author, v_dkim_reason, v_act_insert_x_header_tag, v_filter_type_preflt, 
        v_filter_type_compliance,v_act_decryption,v_act_encryption, v_act_reencryption, v_suspicious, v_dda_status, NULL);
    if not found then
        /*could not insert tb_policy_event*/
        return -1;
    end if; 

    if v_dda_type > 0 and v_dda_type < 10 then
        insert into tb_dtas_mail values (v_msg_id, v_scanner_id, 0, NULL, v_dda_status, '1970-01-01', 0, '1970-01-01', 0, '', '', 
            v_time_date, $AddPolicyEventFromLog31$ || nHourSeq || $AddPolicyEventFromLog3$, v_policy_event_id, v_act_quarantined::bit(1)::integer,
            v_act_archived::bit(1)::integer, v_act_delivery_postponed::bit(1)::integer, v_msg_route);
    end if;

    /*2.add a row into tb_quatantine if v_act_quarantined = 1*/
    if v_act_quarantined = '1' then
        insert into tb_quarantine_$AddPolicyEventFromLog3$ || nHourSeq || $AddPolicyEventFromLog4$ (policy_event_id, 
            msg_id, scanner_id, smtp_sender, smtp_rcpt, msg_subject, msg_route, msg_size, qa_id, rule_name, time_date, 
            filter_type_virus, filter_type_spyware, filter_type_spam, filter_type_attachment,
            filter_type_content, filter_type_size, filter_type_other, filter_type_malformed,
            filter_type_unscanned, filter_type_phishing, attach_name, qt_live_ttl, ref_num, original_msg_id, 
            filter_type_wrs, filter_type_dkim, filter_type_preflt, filter_type_compliance,
            filter_type_suspicious, dda_status, dda_rating) 
            values (v_policy_event_id, v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size,
            v_qa_id, v_rule_name, v_time_date, 
            v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment,
            v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed,
            v_filter_type_unscanned, v_filter_type_phishing, v_attach_name, v_qa_live_ttl, v_ref_num,
            v_org_msg_id, v_filter_type_wrs, v_filter_type_dkim, v_filter_type_preflt, v_filter_type_compliance,
            v_suspicious, v_dda_status, NULL);
        
        if not found then
            /*could not insert tb_quarantine*/
            return -2;
        end if;
    end if;
    
            /*3.add a row into tb_archive if v_act_archived = 1*/
    if v_act_archived = '1' then
        insert into tb_archive_$AddPolicyEventFromLog4$ || nHourSeq || $AddPolicyEventFromLog5$ (policy_event_id, 
            msg_id, scanner_id, smtp_sender, smtp_rcpt, msg_subject, msg_route, msg_size, qa_id, rule_name, time_date, 
            filter_type_virus, filter_type_spyware, filter_type_spam, filter_type_attachment,
            filter_type_content, filter_type_size, filter_type_other, filter_type_malformed,
            filter_type_unscanned, filter_type_phishing, attach_name, qt_live_ttl, original_msg_id, filter_type_wrs, filter_type_dkim, filter_type_preflt, filter_type_compliance,
            filter_type_suspicious, dda_status, dda_rating)
            values (v_policy_event_id, v_msg_id, v_scanner_id, v_smtp_sender, v_smtp_rcpt, v_msg_subject, v_msg_route, v_msg_size,
            v_aa_id, v_rule_name, v_time_date, 
            v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment,
            v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed,
            v_filter_type_unscanned, v_filter_type_phishing, v_attach_name, v_aa_live_ttl,
            v_org_msg_id, v_filter_type_wrs, v_filter_type_dkim, v_filter_type_preflt, v_filter_type_compliance, v_suspicious, v_dda_status, NULL);
        if not found then
            /*could not insert tb_archive*/
            return -3;
        end if;
    end if;

    /*4.add rows into tb_splited_virus_event if v_multi_virus = 1*/
    if v_multi_virus = '1' then
        v_len := length(v_virus_name);
        v_str := substring(v_virus_name from 2 for v_len-1);
        v_pos := position(';' in v_str);
        while v_pos > 0 loop
            insert into tb_splited_virus_event (time_date, virus_name) 
                values (v_time_date, substring(v_str from 1 for (v_pos - 1)));
            v_str := substring(v_str from v_pos + 1 for v_len - v_pos);
            v_len := length(v_str);
            v_pos := position(';' in v_str);
        end loop;
        if trim(both from v_str) <> '' then
            insert into tb_splited_virus_event (time_date, virus_name) values (v_time_date, v_str);
        end if;
    end if;
    
    /*5.add a row into tb_postpone if v_act_delivery_postponed = 1*/
      if v_act_delivery_postponed = '1' then
            insert into tb_postpone values (v_msg_id, v_scanner_id, v_qa_id, v_smtp_sender, v_smtp_rcpt, 
                v_msg_subject, v_msg_route, v_msg_size, v_rule_name, v_time_date, 
                v_filter_type_virus, v_filter_type_spyware, v_filter_type_spam, v_filter_type_attachment,
                v_filter_type_content, v_filter_type_size, v_filter_type_other, v_filter_type_malformed,
                v_filter_type_unscanned, v_filter_type_phishing, v_attach_name, v_qa_live_ttl, v_ref_num,
                v_filter_type_wrs, v_filter_type_dkim, v_filter_type_compliance, v_suspicious, v_dda_status, NULL);
            if not found then
                /*could not insert tb_postpone*/
                return -6;
            end if;    
      end if;


    return 0;
END;
$AddPolicyEventFromLog_body$ LANGUAGE 'plpgsql';$AddPolicyEventFromLog5$ ;
    EXECUTE szCreateTableCmd;
    szCreateTableCmd := $InsertTransac0$CREATE OR REPLACE FUNCTION InsertTransac_$InsertTransac0$ || nHourSeq || $InsertTransac1$(
    in_timestamp text, scan_timestamp text, out_timestamp text, msg_id text, internal_msg_id text, postfix_id text, 
    scanner_id integer, smtp_mail_from text, smtp_rcpt_to text, subject text, client_ip text, 
    delivery_ip text, delivery_feedback text, delivery_status text, act_set text, split_flag integer, 
    extra_item text, todeliverytimestamp text, indeliverytimestamp text)
RETURNS integer AS $InsertTransacBody$
DECLARE
    v_act_deleted               bit := 0;
    v_act_quarantined           bit := 0;
    v_act_delivered             bit := 0;
    v_act_archived              bit := 0;
    v_act_rcptchanged           bit := 0;
    v_act_stamped               bit := 0;
    v_act_attachments_deleted   bit := 0;
    v_act_subjects_tagged       bit := 0;
    v_act_delivery_postponed    bit := 0;
    v_act_notifications_sent    bit := 0;
    v_act_cleaned               bit := 0;
    v_act_bcc                   bit := 0;
    v_act_handoff               bit := 0;
    v_act_insert_x_header_tag   bit := 0;
    v_act_decryption            bit := 0;
    v_act_encryption            bit := 0;
    v_act_reencryption          bit := 0;
    v_msg_tracing_id            INTEGER;
    v_match_policy_flag            BOOLEAN := False;
    v_first_delivery_timestamp  TIMESTAMPTZ := Null;
    v_first_delivery_info       TEXT := Null;
    v_TimeStamp                 TIMESTAMPTZ;
    v_in_timestamp              TEXT := Null;
    v_indeliverytimestamp       TEXT := Null;
    v_HourNum INTEGER;
    v_TableID INTEGER;
    szExecuteCmd text := Null;
    curs refcursor;
    v_client_ip text := Null;
BEGIN
    /*parse to get action */
    if act_set is not Null then
        v_act_deleted            := substring(act_set from 1 for 1);
        v_act_quarantined        := substring(act_set from 2 for 1);
        v_act_delivered          := substring(act_set from 3 for 1);
        v_act_archived           := substring(act_set from 4 for 1);
        v_act_handoff            := substring(act_set from 5 for 1); 
        v_act_rcptchanged        := substring(act_set from 6 for 1);
        v_act_stamped            := substring(act_set from 7 for 1);
        v_act_attachments_deleted:= substring(act_set from 8 for 1);
        v_act_subjects_tagged    := substring(act_set from 9 for 1);
        v_act_delivery_postponed := substring(act_set from 10 for 1);
        v_act_notifications_sent := substring(act_set from 11 for 1);
        v_act_cleaned            := substring(act_set from 12 for 1);
        v_act_bcc                := substring(act_set from 13 for 1);
        v_act_insert_x_header_tag := substring(act_set from 14 for 1);
        v_act_decryption         := substring(act_set from 15 for 1);
        v_act_encryption         := substring(act_set from 16 for 1);
        v_act_reencryption       := substring(act_set from 17 for 1);
        if act_set != '00000000000000000' then
            v_match_policy_flag = True;
        end if;
    end if;
    
      
    if delivery_status = 'deferred' then
        v_first_delivery_timestamp = todeliverytimestamp::TIMESTAMPTZ;
        v_first_delivery_info = delivery_feedback;
    end if;
  
    if (client_ip is null) and (out_timestamp is not Null) and (msg_id is not Null) then
        v_TimeStamp := date_trunc('hour', in_timestamp::TIMESTAMPTZ);
        LOOP
            v_HourNum := (floor(extract(epoch from v_TimeStamp)/3600))::INTEGER % 10000;  
            if (in_timestamp::TIMESTAMPTZ - v_TimeStamp) > INTERVAL'2 hours' then
                EXIT;
            end if;
            RAISE INFO 'v_HourNum:%, v_TimeStamp:%', v_HourNum, v_TimeStamp;    
            v_TimeStamp := v_TimeStamp - INTERVAL'1 hour';
            SELECT INTO v_TableID table_id FROM tb_log_table_info WHERE hour_num = v_HourNum;
            if not found then
                RAISE INFO 'Could not get table_id from tb_log_table_info for timestamp %', v_TimeStamp;
            else
                szExecuteCmd := 'SELECT in_timestamp, client_ip, indeliverytimestamp FROM tb_msg_tracing_' || v_TableID
                                || ' WHERE msg_id = ' || quote_literal(msg_id) 
                                || ' and scan_timestamp is null and delivery_status = ''deferred'';';
                OPEN curs FOR EXECUTE szExecuteCmd;
                FETCH curs INTO v_in_timestamp, v_client_ip, v_indeliverytimestamp;
                if v_in_timestamp is not Null then
                    CLOSE curs;
                    EXIT;   --EXIT the loop when found the record
                end if;
                CLOSE curs;
            end if;
        END LOOP; 
        if v_in_timestamp is not Null and v_client_ip is not Null then     
            --Find the firs the record
            v_TimeStamp := date_trunc('hour', v_in_timestamp::TIMESTAMPTZ);
            v_HourNum := (floor(extract(epoch from v_TimeStamp)/3600))::INTEGER % 10000;  
            SELECT INTO v_TableID table_id FROM tb_log_table_info WHERE hour_num = v_HourNum;
            if not found then
                RAISE INFO 'Could not get table_id from tb_log_table_info for timestamp %', v_TimeStamp;
            else
                szExecuteCmd := 'SELECT InsertTransac_' || v_TableID || '(' 
                                || COALESCE(quote_literal(v_in_timestamp),'null') || ','
                                || COALESCE(quote_literal(scan_timestamp),'null') || ','
                                || COALESCE(quote_literal(out_timestamp),'null') || ','
                                || COALESCE(quote_literal(msg_id),'null') ||','
                                || COALESCE(quote_literal(internal_msg_id),'null') || ','
                                || COALESCE(quote_literal(postfix_id),'null') ||','
                                || quote_literal(scanner_id) || ','
                                || COALESCE(quote_literal(smtp_mail_from),'null') ||','
                                || COALESCE(quote_literal(smtp_rcpt_to),'null') || ','
                                || COALESCE(quote_literal(subject),'null') || ','
                                || COALESCE(quote_literal(v_client_ip),'null') || ','
                                || COALESCE(quote_literal(delivery_ip),'null') || ','
                                || COALESCE(quote_literal(delivery_feedback),'null') || ','
                                || COALESCE(quote_literal(delivery_status),'null') || ','
                                || COALESCE(quote_literal(act_set),'null') || ','
                                || quote_literal(split_flag) || ','
                                || COALESCE(quote_literal(extra_item),'null') || ','
                                || COALESCE(quote_literal(todeliverytimestamp),'null') || ','
                                || COALESCE(quote_literal(v_indeliverytimestamp),'null') || ');';
                EXECUTE szExecuteCmd;
                if not found then
                    RAISE INFO 'select InsertTransac_% failed. szExecuteCmd = %', v_TableID, szExecuteCmd;
                    return -1;
                end if;    
                return 0;             
            end if;             
        else
            RAISE INFO 'Cannot got the first deferred record in_timestamp %', v_in_timestamp;
        end if;
    end if;    
    
    v_msg_tracing_id := nextval('tb_msg_tracing_$InsertTransac1$ || nHourSeq || $InsertTransac2$_msg_tracing_id_seq');        
    INSERT INTO tb_msg_tracing_$InsertTransac2$ || nHourSeq || $InsertTransac3$ VALUES (v_msg_tracing_id, 
                                        in_timestamp::timestamptz, 
                                        scan_timestamp::timestamptz, 
                                        out_timestamp::timestamptz,
                                        v_first_delivery_timestamp,
                                        msg_id,
                                        internal_msg_id, 
                                        postfix_id,
                                        scanner_id,
                                        smtp_mail_from,
                                        smtp_rcpt_to,
                                        subject,
                                        client_ip,
                                        delivery_ip,
                                        delivery_feedback,
                                        delivery_status,
                                        v_first_delivery_info,
                                        v_act_deleted,
                                        v_act_quarantined,
                                        v_act_delivered,
                                        v_act_archived,
                                        v_act_rcptchanged,
                                        v_act_stamped,
                                        v_act_attachments_deleted,
                                        v_act_subjects_tagged,
                                        v_act_delivery_postponed,
                                        v_act_notifications_sent,
                                        v_act_cleaned,
                                        v_act_bcc,
                                        v_act_handoff,
                                        v_act_insert_x_header_tag,
                                        extra_item,
                                        split_flag,
                                        False,
                                        v_match_policy_flag,
                                        todeliverytimestamp::timestamptz, 
                                        indeliverytimestamp::timestamptz,
                                        v_act_decryption,
                                        v_act_encryption,
                                        v_act_reencryption);
    if not found then
        /*could not insert tb_msg_tracing*/
        return -1;
    else
        return v_msg_tracing_id;
    end if;                                          
END;
$InsertTransacBody$
LANGUAGE $sql$plpgsql$sql$;$InsertTransac3$;
    EXECUTE szCreateTableCmd;

    nTmp := nTmp + 1;
    end loop;

    return 0;
END;
$createBody$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION RollbackLogTables() RETURNS void AS ' 
DECLARE
        tbl             RECORD;
        cnt             INTEGER;
BEGIN
        FOR tbl IN EXECUTE ''SELECT COUNT(routine_name) as cnt FROM information_schema.routines WHERE routine_name like ''''addpolicyeventfromlog_%'''' '' LOOP
                cnt := tbl.cnt;
        END LOOP;

        FOR tbl IN EXECUTE ''SELECT DISTINCT routine_name FROM information_schema.routines WHERE routine_name like ''''addpolicyeventfromlog_%'''' '' LOOP
                EXECUTE ''DROP FUNCTION '' || tbl.routine_name || ''(bpchar, int4, varchar, varchar, varchar, int2, float8, varchar, timestamptz, bpchar, varchar, bpchar, float4, int4, int2, varchar, varchar, varchar, varchar, int4, timestamptz, int4, timestamptz, varchar, bit, int4, int4, int4, varchar, varchar, varchar,int4)'';
        END LOOP;
     
        EXECUTE ''SELECT CreateLogImportFunc(0, '' || cnt || '')'';
     
        return;
END;
' LANGUAGE plpgsql;

SELECT RollbackLogTables();
DROP FUNCTION RollbackLogTables();
