#! /bin/bash
trap "" SIGINT
trap "unlock" EXIT
imss_work_dir=/opt/trend/imss
ippro_work_dir=/opt/trend/ipprofiler
nrs_work_dir=/opt/trend/nrs
IMSS_HOME=/opt/trend/imss
LD_LIBRARY_PATH=$LD_LIBRARY_PATH:$IMSS_HOME/lib:/opt/TrendMicro/GoldenGate/lib
export PATH=$PATH:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/opt/trend/imss/script:/opt/trend/imss/postfix/usr/sbin	# work around to fix bug IMSVA 8.0-00300, env variable PATH is lost when called from clish
export PATH=$PATH:/opt/TrendMicro/GoldenGate/bin
export IMSS_HOME
export LD_LIBRARY_PATH
#adminDB
DBSERVER="127.0.0.1"
DBNAME=imss
DBTYPE="POSTGRESQL"
DBPORT="5432"
DBUSER=sa
DBUSERPASS=""
DBUSERPASS_ENC=""
export DBSERVER
export DBNAME
export DBTYPE
export DBPORT
export DBUSER
export DBUSERPASS
export DBUSERPASS_ENC
ODBCSYSINI=$IMSS_HOME/config/
ODBCINI=$IMSS_HOME/config/odbc.ini
DBINI=$IMSS_HOME/config/database.ini
IMSS_DB_DIR=postgresql
IMSS_DB_INI=database.ini
export ODBCSYSINI
export ODBCINI
export DBINI
export IMSS_DB_DIR
export IMSS_DB_INI
TESTDB=$IMSS_HOME/bin/testdb
EUQOP=/opt/trend/imss/bin/euqop
PSQL=$IMSS_HOME/PostgreSQL/bin/psql
PS="ps -ef --width 1000"
## Entry
# IMSA box is only for en_US, so hardcode it.
#ORG_LANG=`locale | grep LANG | awk -F= '{print $2}'`
#ORG_LCALL=`locale | grep LC_ALL | awk -F= '{print $2}'`
ORG_LANG=en_US
ORG_LCALL=en_US
LANG=""
LC_ALL=""
export LANG
export LC_ALL
. $IMSS_HOME/script/imssfunctions
lock
id | grep root > /dev/null 2>&1
if [ $? -ne 0 ] ; then
   echo "You must have a root privilege to execute this script."
   exit 253
fi
if ps -ef | grep imssctl | grep -v grep > /dev/null ; then
   echo "Some component of IMSS is starting up or shutting down."
   echo "Try again later."
   exit 252
fi

EncryptStr()
{
    str=$1
    $IMSS_HOME/bin/passwd_util -e $str
}

StartApp()
{
   pname="$1"
   sname="$2"
   if test ! -f "$IMSS_HOME/script/$sname"; then
      return
   fi
   $PS | grep "$pname" | grep -v grep > /dev/null
   if  [ $? != 0 ] ; then
      $IMSS_HOME/script/$sname start
   else
      $IMSS_HOME/script/$sname restart
   fi
}
StopApp()
{
   pname="$1"
   sname="$2"
   pidname="$1.pid"
   if test ! -f "$IMSS_HOME/script/$sname"; then
      return
   fi
   $PS | grep "$pname" | grep -v grep > /dev/null
   if [ $? = 0 ] ; then
      $IMSS_HOME/script/$sname stop
      rm -f $IMSS_HOME/bin/$pidname
   fi
}
StartAllApp()
{
   LANG="$ORG_LANG"
   LC_ALL="$ORG_LCALL"
   export LANG
   export LC_ALL
   StartApp imsscmagent S99CMAGENT
   if [ "$PG_DB_SERVER" = "127.0.0.1" ]; then
      $IMSS_HOME/UI/adminUI/bin/Tomcat.sh start
      $IMSS_HOME/script/S99SCHEDULED start >/dev/null 2>&1
      $IMSS_HOME/bind/named -u named
      $IMSS_HOME/script/S99FOXDNS start
   fi
   StartApp imssmgrmon S99MONITOR
   LANG=""
   LC_ALL=""
   export LANG
   export LC_ALL
}
setup_db_env_imss()
{
   work_dir="$1"
   export IMSS_HOME=$work_dir
   export LD_LIBRARY_PATH=$IMSS_HOME/lib
   export ODBCINI=$IMSS_HOME/config/odbc.ini
   export ODBCSYSINI=$IMSS_HOME/config
   export DBINI=$IMSS_HOME/config/database.ini
    
   DBSERVER=`grep "Servername" $ODBCINI | awk -F'=' '{print $2}'`
   DBUSER=`grep "UserId" $DBINI | awk -F= '{print $2}'`
   DBUSERPASS=`grep "Password" $DBINI | awk -F= '{print $2}'`
   DBNAME=`grep "Database=" $DBINI | awk -F= '{print $2}'`

   DBUSERPASS_ENC=`EncryptStr $DBUSERPASS`
}
backup_dbconf()
{
   work_dir="$1"
   cp $work_dir/config/odbcinst.ini $work_dir/config/odbcinst.ini.org
   cp $work_dir/config/odbc.ini $work_dir/config/odbc.ini.org
   cp $work_dir/config/database.ini $work_dir/config/database.ini.org
}
restore_dbconf()
{
   work_dir="$1"
   cp $work_dir/config/odbcinst.ini.org $work_dir/config/odbcinst.ini
   cp $work_dir/config/odbc.ini.org $work_dir/config/odbc.ini
   cp $work_dir/config/database.ini.org $work_dir/config/database.ini
}
remove_bkconf()
{
   work_dir="$1"
   rm $work_dir/config/odbcinst.ini.org
   rm $work_dir/config/odbc.ini.org
   rm $work_dir/config/database.ini.org
}

use_tmp_odbc_ini()
{
	NEWADDR=$1
	TMP_ODBC="/tmp/odbc.ini"
	sed -e "s/Servername.*/Servername=$NEWADDR/g" $IMSS_HOME/config/odbc.ini >$TMP_ODBC
	export ODBCINI=$TMP_ODBC
}

# changed by tonylow 1201
check_admin_db_use_tmp_odbc()
{
   PG_DB_SERVER="$1"
   PG_DB_NAME=imss
   PG_DB_USER="$2"
   if [ "$PG_DB_USER" = "" ]; then
      PG_DB_USER="sa"
   fi
   PG_DB_PASS="$3"

   PG_DB_PASS_ENC=`EncryptStr $PG_DB_PASS`

	use_tmp_odbc_ini $PG_DB_SERVER
	$TESTDB 1 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME"
   if [ $? -eq 0 ]; then
      echo "DB does exist."
      DBEXIST="YES"
   else
      echo "DB doesn't exist."
      DBEXIST="NO"
   fi	
   export ODBCINI=$IMSS_HOME/config/odbc.ini
}

# tonylow 1201: do not use this any more!
CheckAdminDB()
{
   PG_DB_SERVER="$1"
   PG_DB_NAME=imss
   PG_DB_USER="$2"
   if [ "$PG_DB_USER" = "" ]; then
      PG_DB_USER="sa"
   fi
   PG_DB_PASS="$3"
   PG_DB_PASS_ENC=`EncryptStr $PG_DB_PASS`

   setup_db_env_imss $imss_work_dir
   backup_dbconf $imss_work_dir

   sed -e "s,_PKG_INSTALL_ROOT_,$IMSS_HOME,g" $IMSS_HOME/config/odbcinst.ini.tmpl > $IMSS_HOME/config/pp$$
   mv $IMSS_HOME/config/pp$$ $IMSS_HOME/config/odbcinst.ini
   sed -e "s,_PKG_SERVER_,$PG_DB_SERVER,g" -e "s,_PKG_DB_,$PG_DB_NAME,g" -e "s,_PKG_USER_,$PG_DB_USER,g" -e "s,_PKG_PASS_,$PG_DB_PASS_ENC,g" $IMSS_HOME/config/odbc.ini.tmpl > $IMSS_HOME/config/pp$$
   mv $IMSS_HOME/config/pp$$ $IMSS_HOME/config/odbc.ini
   sed -e "s,_PKG_DB_,$PG_DB_NAME,g" -e "s,_PKG_USER_,$PG_DB_USER,g" -e "s,_PKG_PASS_,$PG_DB_PASS_ENC,g" $IMSS_HOME/config/database.ini.tmpl > $IMSS_HOME/config/pp$$
   mv $IMSS_HOME/config/pp$$ $IMSS_HOME/config/database.ini
   $TESTDB 1 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME"
   if [ $? -eq 0 ]; then
      echo "DB does exist."
      DBEXIST="YES"
   else
      echo "DB doesn't exist."
      DBEXIST="NO"
   fi
   
   cp -f /opt/trend/imss/config/odbc.ini /tmp
   cp -f /opt/trend/imss/config/database.ini /tmp
   restore_dbconf $imss_work_dir
}

getScannerID()
{
   configdir="$1"/config
   SCANNERID=`grep "scanner_id" $configdir/imss.ini 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`
   if [ "$SCANNERID" = "" ]; then
      SCANNERID=0
   fi
}
getDBInfo()
{
   CONFIGDIR="$1"/config
   DBSERVER=`grep Servername $CONFIGDIR/odbc.ini | awk -F'=' '{print $2}'`
   DBNAME=`grep Database $CONFIGDIR/odbc.ini | awk -F'=' '{print $2}'`
   DBUSER=`grep UserName $CONFIGDIR/odbc.ini | awk -F'=' '{print $2}'`
   DBUSERPASS=`grep Password $CONFIGDIR/odbc.ini | awk -F'=' '{print $2}'`
   DBUSERPASS_ENC=`EncryptStr $DBUSERPASS`
}
Replace_DB_INFO()
{
   configdir="$1"
   if [ -f $configdir/odbcinst.ini ]; then
      sed -e "s,_PKG_INSTALL_ROOT_,$IMSS_HOME,g" $IMSS_HOME/config/odbcinst.ini.tmpl > $configdir/pp$$
      cp $configdir/pp$$ $configdir/odbcinst.ini
      rm -f $configdir/pp$$
   fi
   if [ -f $configdir/odbc.ini ]; then
      sed -e "s,_PKG_SERVER_,$PG_DB_SERVER,g" -e "s,_PKG_DB_,$PG_DB_NAME,g" -e "s,_PKG_USER_,$PG_DB_USER,g" -e "s,_PKG_PASS_,$PG_DB_PASS_ENC,g" $IMSS_HOME/config/odbc.ini.tmpl > $configdir/pp$$
      cp $configdir/pp$$ $configdir/odbc.ini
      rm -f $configdir/pp$$
   fi
   if [ -f $configdir/database.ini ]; then
      sed -e "s,_PKG_DB_,$PG_DB_NAME,g" -e "s,_PKG_USER_,$PG_DB_USER,g" -e "s,_PKG_PASS_,$PG_DB_PASS_ENC,g" $IMSS_HOME/config/database.ini.tmpl > $configdir/pp$$
      cp $configdir/pp$$ $configdir/database.ini
      rm -f $configdir/pp$$
   fi
}

register()
{
   PG_DB_SERVER="$1"
   PG_DB_NAME=imss
   PG_DB_USER="$2"
   if [ "$PG_DB_USER" = "" ]; then
      PG_DB_USER="sa"
   fi
   PG_DB_PASS="$3"
   PG_DB_PASS_ENC=`EncryptStr $PG_DB_PASS`
   
   getScannerID $imss_work_dir
   if [ "$SCANNERID" -ne 1 ]; then
      echo "This server can not be register because the scanner_id is not equal to 1."
      exit 2
   fi
   getDBInfo $imss_work_dir
   if [ "$DBSERVER" != "127.0.0.1" ]; then
      echo "This server can not be register because the DBSERVER is not 127.0.0.1."
      exit 3
   fi
   #child_server_count=`grep "child_server_count" $imss_work_dir/config/imss.ini.db 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`
   #if [ "$child_server_count" = "" ]; then
   #   child_server_count=0
   #fi
   #if [ "$child_server_count" -ne 0 ]; then
   #   echo "This server can not be register because it is a Parent server with registered Child server."
   #   exit 4
   #fi
   # changed by tonylow 1201
   #CheckAdminDB "$1" "$2" "$3"
   check_admin_db_use_tmp_odbc "$PG_DB_SERVER" "$PG_DB_USER" "$PG_DB_PASS"
   if [ "$DBEXIST" = "NO" ]; then
      echo "This server can not be register because can not access to Admin DB."
      #remove_bkconf $imss_work_dir
      exit 1
   fi

   # tonylow 1212: fix IMSA-7.0-00602
   # deregister CMAgent before register to Parent server
   /opt/trend/imss/script/S99CMAGENT isregistered
   # if cmagent registered, unregister it
   if [ $? -eq 0 ] ; then
        /opt/trend/imss/script/S99CMAGENT unregister
   fi

   /opt/trend/imss/script/imssctl.sh stop_others
   /opt/trend/imss/script/S99SCHEDULED stop
   /opt/trend/imss/script/S99UPDATE stop
   
   # remove EUQ DB data (don't care about error)
   /opt/trend/imss/script/euqtrunc.sh

   # the realaddress is different in IMSA box
   if [ -f /etc/conf/ggb.conf ]; then
      REALADDR=`/opt/TrendMicro/GoldenGate/bin/glcfg /etc/conf/ggb.conf val data_port ip`
   else
      REALADDR=`ifconfig -a|grep -v "127.0.0.1"|grep inet|awk -F' ' '{print $2}'|awk -F: '{print $2}' |head -1`
   fi

   sed -e "s,scanner_id=1,scanner_id=0,g" $IMSS_HOME/config/imss.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/imss.ini
   rm -f $IMSS_HOME/temp/pp$$

   # changed replace dbinfo to use tmp odbc by tonylow 1201
   use_tmp_odbc_ini $PG_DB_SERVER
   
   # Fei: check if local ip exists in Parent's DB, if exists, return error
   $TESTDB 105 IMSS sa "/P $PG_DB_PASS" imss "$REALADDR" > /dev/null 2>&1
   if [ $? -ne 5 ] ; then
      export ODBCINI=$IMSS_HOME/config/odbc.ini
      REG_RESULT=="NO"
      echo "The service components can not register to $PG_DB_SERVER."

      # added by tonylow 1201: fixed IMSA-7.0-00052

      sed -e "s,scanner_id=0,scanner_id=1,g" $IMSS_HOME/config/imss.ini > $IMSS_HOME/temp/pp$$
      cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/imss.ini
      rm -f $IMSS_HOME/temp/pp$$

      /opt/trend/imss/script/imssctl.sh start

      #PG_DB_SERVER="127.0.0.1"
      #Replace_DB_INFO $IMSS_HOME/config
      #restore_dbconf $imss_work_dir
      #remove_bkconf $imss_work_dir
      exit 11
   fi

   STATUS=0202010201
   $TESTDB 10 IMSS sa "/P $PG_DB_PASS" imss "$REALADDR" "$STATUS" >/dev/null 2>&1
   if [ $? -eq 0 ]; then
      export ODBCINI=$IMSS_HOME/config/odbc.ini
      REG_RESULT=="YES"
      echo "The service components have been registered to $PG_DB_SERVER successfully."
   else
      export ODBCINI=$IMSS_HOME/config/odbc.ini
      REG_RESULT=="NO"
      echo "The service components can not register to $PG_DB_SERVER."

      # added by tonylow 1201: fixed IMSA-7.0-00052

      sed -e "s,scanner_id=0,scanner_id=1,g" $IMSS_HOME/config/imss.ini > $IMSS_HOME/temp/pp$$
      cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/imss.ini
      rm -f $IMSS_HOME/temp/pp$$

      /opt/trend/imss/script/imssctl.sh start

      #PG_DB_SERVER="127.0.0.1"
      #Replace_DB_INFO $IMSS_HOME/config
      #restore_dbconf $imss_work_dir
      #remove_bkconf $imss_work_dir
      exit 5
   fi

   Replace_DB_INFO $IMSS_HOME/config
   Replace_DB_INFO $ippro_work_dir/config
   Replace_DB_INFO $nrs_work_dir

   SCANNERID=`grep "scanner_id" $IMSS_HOME/config/imss.ini 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`

   sed -e "s,scanner_id=1,scanner_id=$SCANNERID,g" $IMSS_HOME/config/trace.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/trace.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,scanner_id=1,scanner_id=$SCANNERID,g" $ippro_work_dir/config/foxproxy.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $ippro_work_dir/config/foxproxy.ini
   rm -f $IMSS_HOME/temp/pp$$
   sed -e "s,scanner_id=1,scanner_id=$SCANNERID,g" $nrs_work_dir/logparser.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $nrs_work_dir/logparser.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,127.0.0.1,$PG_DB_SERVER,g" $ippro_work_dir/config/fox_dns_server.list > $ippro_work_dir/config/pp$$
   cp $ippro_work_dir/config/pp$$ $ippro_work_dir/config/fox_dns_server.list
   rm -f $ippro_work_dir/config/pp$$

   # tonylow 1215: IMSA-7.0-00686 -- Disable for fixing bug IMSA-7.0-01357
   #postconf -e "smtpd_client_restrictions ="
   #postconf -e "rbl_reply_maps ="
   #rm -f /opt/trend/imss/postfix/etc/postfix/imss_rbl_reply*

   # now register the EUQ
   /opt/trend/imss/bin/euqop reg $REALADDR
   # do not return failed because slave components has been registered!
   #if [ $? -ne 0 ]; then
   #   exit 6
   #fi
   
   echo "master_ip=$PG_DB_SERVER" > $IMSS_HOME/config/slave.nfo
   
   # remove the log, reports and quarantined & archived mail.
   rm -rf $IMSS_HOME/UI/adminUI/ROOT/reports/*
   rm -rf $IMSS_HOME/log/*
    
   # IMSA-7.0-00356, if mail count is too large, rm may fail.
   #rm -rf $IMSS_HOME/queue/quarantine/*/*/*
   #rm -rf $IMSS_HOME/queue/archive/*/*/*
   #rm -rf $IMSS_HOME/queue/quarantine
   #rm -rf $IMSS_HOME/queue/archive
   # IMSA-7.0-01276, need many time to purge several giga bytes data
   # Solution: mv the folder to older folder, and remove it in backend
   if [ ! -d /var/queue_temp/ ] ; then
       mkdir /var/queue_temp/
   fi
   # get inode of $IMSS_HOME/queue/quarantine
   INODE=`ls -li $IMSS_HOME/queue/ 2>/dev/null|grep quarantine|awk '{ print $1 }'`
   TIME=`date +%y%m%d%H%M%S%N`
   mv $IMSS_HOME/queue/quarantine /var/queue_temp/quarantine_${INODE}_${TIME}
   # get inode of $IMSS_HOME/queue/archive
   INODE=`ls -li $IMSS_HOME/queue/ 2>/dev/null|grep archive|awk '{ print $1 }'`
   TIME=`date +%y%m%d%H%M%S%N`
   mv $IMSS_HOME/queue/archive /var/queue_temp/archive_${INODE}_${TIME}
   /opt/trend/imss/bin/close_handle_sh "nohup rm -rf /var/queue_temp/ > /dev/null 2>&1 &"
   
   ENUM_HEX="0 1 2 3 4 5 6 7 8 9 A B C D E F"
   for folder_name in ${ENUM_HEX} ; do
      for folder_name2 in ${ENUM_HEX}; do
         mkdir -p $IMSS_HOME/queue/quarantine/$folder_name/$folder_name2
         mkdir -p $IMSS_HOME/queue/archive/$folder_name/$folder_name2
      done
   done
   
   chown -R imss $IMSS_HOME/queue/quarantine
   chgrp -R imss $IMSS_HOME/queue/quarantine
   chown -R imss $IMSS_HOME/queue/archive
   chgrp -R imss $IMSS_HOME/queue/archive
   
   # update the firewall setting, just add the parent's IP to the list.
   
   IMLic=`grep "cszLicProfileFile = " $imss_work_dir/config/imss.ini.db 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`
   if [ "$IMLic" = " " ]; then
      IMLic=`grep "cszLicProfileFile2 = " $imss_work_dir/config/imss.ini.db 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`
   fi
   if [ "$IMLic" != " " ]; then
      /opt/trend/imss/script/S99IMSS start
      sleep 1
      STATUS=0303030302
      $TESTDB 10 IMSS sa "/P $PG_DB_PASS" imss "$REALADDR" "$STATUS" >/dev/null 2>&1
      sleep 1
   fi
   /opt/trend/imss/script/imssctl.sh start
   
   exit 0
}

apply_app_patch()
{
    # mount hidden partition
    #mount /dev/sda6 /mnt/backup > /dev/null 2>&1
    #if [ $? -ne 0 ] ; then
    #    umount /dev/sda6 > /dev/null 2>&1
    #    mount /dev/sda6 /mnt/backup > /dev/null 2>&1
    #    if [ $? -ne 0 ] ; then
    #        echo "Cannot mount image partition, apply application patch failed."
    #        return 1
    #    fi
    #fi
    
    FILE_CONF=/mnt/backup/patches/patches.history
    
    if [ -f $FILE_CONF ] ; then
        len=`wc -l < $FILE_CONF`
        done_file_cnt=0
        if [ $len -ne 0 ] ; then
            echo "Applying local patch..."
            for i in `cat $FILE_CONF | sort -u` ; do
                # TODO: need to trim $f
                if [ "$i" != "" ] ; then
                    f=/mnt/backup/patches/$i
                    if [ -r "/$f" ] ; then
                        done_file_cnt=`expr ${done_file_cnt} + 1`
                        rm -rf /opt/trend/imss/patch/temp
                        mkdir /opt/trend/imss/patch /opt/trend/imss/patch/temp > /dev/null 2>&1
                        tar zxvf $f -C /opt/trend/imss/patch/temp > /dev/null 2>&1
                        PATCH_SH=/opt/trend/imss/patch/temp/apply.sh
                        if [ -f $PATCH_SH ] ; then
                            chmod +x /opt/trend/imss/patch/temp/*.sh
                            $PATCH_SH unreg >> /opt/trend/imss/patch/patchlog.txt 2>>/opt/trend/imss/patch/patchlog.txt
                        fi
                        sleep 1
                    fi
                fi
            done
        else
            echo "No patch history."
        fi
    else
        echo "No patch history."
    fi
    #umount /dev/sda6
    return 0
}
rescue_with_patch()
{
    # rescue
    /etc/init.d/rcAppRescue >/dev/null 2>/dev/null
    
    # app patch
    apply_app_patch
    exit $?
}
# this is for rescue from CLI and WebUI
deregister_rescue()
{
   getScannerID $imss_work_dir
   if [ "$SCANNERID" -eq 1 ]; then
      echo "This server can not be deregister because the scanner_id is equal to 1."
      exit 6
   fi
   getDBInfo $imss_work_dir
   if [ '$DBSERVER' = "127.0.0.1" ]; then
      echo "This server can not be deregister because the DBSERVER is 127.0.0.1."
      exit 7
   fi
   PG_DB_SERVER="$DBSERVER"
   PG_DB_NAME=imss
   PG_DB_USER="$DBUSER"
   PG_DB_PASS="$DBUSERPASS"
   PG_DB_PASS_ENC=`EncryptStr $PG_DB_PASS`
   setup_db_env_imss $imss_work_dir
   $TESTDB 1 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME"
   if [ $? -eq 0 ]; then
      DBEXIST="YES"
   else
      DBEXIST="NO"
      echo "IMSA is unable to remove this child's information from the parent, as it has been unregistered forcibly." 1>&2
      echo "Please unregister this child from the Admin UI to remove the child's information later." 1>&2
      #exit 1
   fi
   
   rescue_with_patch
   
   if [ "$DBEXIST" = "YES" ]; then
      # Unregister EQU 
	  $EUQOP unreg $SCANNERID  2>/dev/null
	  $TESTDB 11 IMSS sa "/P $PG_DB_PASS" imss
	  # Must restart the all the imssmgr to apply the trusted IP change.
	  $EUQOP restartmgr 2>/dev/null
   fi	   
   # No need update remove the child's ip from trusted ip list. Since if user unregister the child, it will 
   # remove the IP from trusted IP list.
	  
   # No need update the firewall settings here, since it is handled in rescue script.

   exit 0
}
# this function only for rollback when register failed
deregister()
{
   getScannerID $imss_work_dir
   if [ "$SCANNERID" -eq 1 ]; then
      echo "This server can not be deregister because the scanner_id is equal to 1."
      exit 6
   fi

   getDBInfo $imss_work_dir
   if [ '$DBSERVER' = "127.0.0.1" ]; then
      echo "This server can not be deregister because the DBSERVER is 127.0.0.1."
      exit 7
   fi

   PG_DB_SERVER="$DBSERVER"
   PG_DB_NAME=imss
   PG_DB_USER="$DBUSER"
   PG_DB_PASS="$DBUSERPASS"
   PG_DB_PASS_ENC=`EncryptStr $PG_DB_PASS`
   setup_db_env_imss $imss_work_dir
   
   $TESTDB 1 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME"
   if [ $? -eq 0 ]; then
      DBEXIST="YES"
   else
      DBEXIST="NO"
      echo "This server can not be deregister because can not access to Admin DB."
      exit 1
   fi

   # force deregister euq from master
   $IMSS_HOME/bin/euqop unreg $SCANNERID
   
   $TESTDB 11 IMSS sa "/P $PG_DB_PASS" imss

   /opt/trend/imss/script/imssctl.sh stop_others
   /opt/trend/imss/script/S99SCHEDULED stop
   /opt/trend/imss/script/S99UPDATE stop

   rm $IMSS_HOME/config/slave.nfo

   sed -e "s,scanner_id=0,scanner_id=1,g" $IMSS_HOME/config/imss.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/imss.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,scanner_id=$SCANNERID,scanner_id=1,g" $IMSS_HOME/config/trace.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $IMSS_HOME/config/trace.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,scanner_id=$SCANNERID,scanner_id=1,g" $ippro_work_dir/config/foxproxy.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $ippro_work_dir/config/foxproxy.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,scanner_id=$SCANNERID,scanner_id=1,g" $nrs_work_dir/logparser.ini > $IMSS_HOME/temp/pp$$
   cp $IMSS_HOME/temp/pp$$ $nrs_work_dir/logparser.ini
   rm -f $IMSS_HOME/temp/pp$$

   sed -e "s,$PG_DB_SERVER,127.0.0.1,g" $ippro_work_dir/config/fox_dns_server.list > $ippro_work_dir/config/pp$$
   cp $ippro_work_dir/config/pp$$ $ippro_work_dir/config/fox_dns_server.list
   rm -f $ippro_work_dir/config/pp$$

   PG_DB_SERVER="127.0.0.1"
   Replace_DB_INFO $IMSS_HOME/config
   Replace_DB_INFO $ippro_work_dir/config
   Replace_DB_INFO $nrs_work_dir

   $IMSS_HOME/script/dbctl.sh start
   StartAllApp

   exit 0
}

change_server_address()
{
   NEWADDR=$1
   if [ "$NEWADDR" = "" ]; then
      return 1;
   fi
   
   FILES="$IMSS_HOME/config/odbc.ini $ippro_work_dir/config/odbc.ini $nrs_work_dir/odbc.ini" 
   
   for i in $FILES
   do
      FILE=$i
      if [ -f $FILE ]; then
         cp -f $FILE $FILE.bak 
         sed -e "s/Servername.*/Servername=$NEWADDR/g" $FILE.bak >$FILE
         rm -f $FILE.bak
      fi 
   done 
   
   # Update fox_dns_server.list
   echo "Update fox_dns_server.list"
   CURADDR=`cat /opt/trend/imss/config/slave.nfo | awk -F= '{print $2}'`
   sed -e "s,$CURADDR,$NEWADDR,g" $ippro_work_dir/config/fox_dns_server.list > $ippro_work_dir/config/pp$$
   cp $ippro_work_dir/config/pp$$ $ippro_work_dir/config/fox_dns_server.list
   rm -f $ippro_work_dir/config/pp$$
   $ippro_work_dir/script/foxproxyd status | grep -i running >/dev/null 2>&1
   if [ $? -eq 0 ]; then
       echo "Restart foxproxy daemon"
       $ippro_work_dir/script/foxproxyd restart
   fi

   echo "$NEWADDR	53" >$ippro_work_dir/config/normal_dns_server.list
   echo "master_ip=$NEWADDR" > $IMSS_HOME/config/slave.nfo

}

change_master()
{
   NEWADDR=$1
   #must check whether the IP is same as the registerred one.
   OLDADDR=`cat $IMSS_HOME/config/slave.nfo`
   
   if [ "master_ip=$NEWADDR" = $OLDADDR ]; then
      echo "Change to the same parent, so do nothing"
      exit 1
   fi
   
   getDBInfo $imss_work_dir
   # changed by tonylow 1201
   check_admin_db_use_tmp_odbc $NEWADDR sa '!CRYPT!20DC0860772097D98ABC2A17D69103AF10B097D5511'
   if [ "$DBEXIST" = "NO" ]; then
      echo "This server can not be register because can not access to Admin DB."
      exit 2
   fi
   getScannerID $imss_work_dir
   if [ "$SCANNERID" -eq 1 ]; then
      echo "The ip of this server can not be changed because the scanner_id equals to 1."
      exit 3
   fi
   
   # Compare the scanner id in imss.ini with that in DB, if they are different with each other,
   # it means it change to another parent, just terminate.
   SCANNERID=`grep "scanner_id" $IMSS_HOME/config/imss.ini 2>/dev/null|grep -v "^#" | awk -F= '{print $2}'`
   
   # Get the IP address of the box
   if [ -f /etc/conf/ggb.conf ]; then
      REALADDR=`/opt/TrendMicro/GoldenGate/bin/glcfg /etc/conf/ggb.conf val data_port ip`
   fi
   
   # Check IP firstly, if ip is same, just update the settings. 
   //The checkDB copy the odbc.ini to the tmp, and use the temp file for db connect.
   export ODBCINI=/tmp/odbc.ini
   
   IP_IN_DB=`$IMSS_HOME/bin/testdb 101 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME" $SCANNERID`
   if [ $? -eq 0 ]; then
      if [ "$IP_IN_DB" = "$REALADDR" ]; then
         # the IP in DB is same as child's ip
         echo "The IP is same as that in DB, it is OK to change the parent's IP."
         /opt/trend/imss/script/imssctl.sh stop
         change_server_address $NEWADDR   
         # No need to handle the EUQ, since ConfgMgr.cpp handle it.
         /opt/trend/imss/script/imssctl.sh start
         exit 0
      fi
   fi 
   
   MAC_ADDR=`ifconfig eth0 |grep HWaddr | awk -F' '  '{print $5}'`
   MAC_IN_DB=`$IMSS_HOME/bin/testdb 102 IMSS "$PG_DB_USER" "/P $PG_DB_PASS" "$PG_DB_NAME" $SCANNERID`
   if [ $? -eq 0 ]; then
      if [ "$MAC_IN_DB" = "$MAC_ADDR" ]; then
         # the Name in DB is same as child's ip
         echo "The name is same as that in DB, it is OK to change the parent's IP."
         /opt/trend/imss/script/imssctl.sh stop
         change_server_address $NEWADDR   
         # No need to handle the EUQ, since ConfgMgr.cpp handle it.
         /opt/trend/imss/script/imssctl.sh start
         exit 0
      fi
   fi 
   # no matter the db return error, or the name is different for the same scanner ID, it means the child 
   # is changed to a different parent, so return error;
   exit 4   
}
get_db_svr_ip()
{
   getDBInfo $imss_work_dir
   echo "$DBSERVER"
   exit 0
}
get_child_osapp_ver()
{
   #OSVER=`uname -r | cut -d '-' -f 2 | cut -d '.' -f 1-2`
    OSVER=`uname -r |cut -d 'A' -f2 |cut -c 2- | cut -d '.' -f 1-2`
   APPVER=`/opt/trend/imss/script/S99IMSS version | cut -d '-' -f 1 | cut -d ' ' -f 2`
   APPBUILDNUM=`/opt/trend/imss/script/S99IMSS version | cut -d ' ' -f 2 | cut -d '_' -f 3`
   echo "$OSVER $APPVER $APPBUILDNUM"
   exit 0
}
get_parent_osapp_ver()
{
   setup_db_env_imss $imss_work_dir
	use_tmp_odbc_ini $1
	$TESTDB 104 IMSS "$2" "/P $3" imss
   export ODBCINI=$IMSS_HOME/config/odbc.ini
   exit 0
}
ggb_cfg()
{
   /opt/TrendMicro/GoldenGate/bin/glcfg /etc/conf/ggb.conf "$1" "$2" "$3" $4
   ret=$?
   echo
   exit $ret
}
apply_network()
{
   #/opt/trend/imss/script/S99MONITOR stop
   #/opt/trend/imss/script/S99ADMINUI stop
   #/opt/trend/imss/script/imssctl.sh stop_others
   /etc/init.d/rcNetwork reload
   #/opt/trend/imss/script/dbctl.sh reload
   #/opt/trend/imss/script/imssctl.sh start
   #/opt/trend/imss/script/S99ADMINUI start &
   exit 0
}
restart_imss()
{
   /opt/trend/imss/bin/close_handle_sh "/opt/trend/imss/script/imssctl.sh restart" &
   exit 0
}
restart_imss_noui()
{
   /opt/trend/imss/script/imssctl.sh stop_others
   /opt/trend/imss/script/dbctl.sh reload
   /opt/trend/imss/bin/close_handle_sh "/opt/trend/imss/script/imssctl.sh start"
   exit 0
}

# It will try to connect the AdminDB, if success, update the trusted IP list.
change_ip()
{
   $IMSS_HOME/script/is_master.sh
   if [ $? -eq 1 ]; then
      exit 0
   fi

   NEWIPADDR=$1
   setup_db_env_imss  $imss_work_dir
   
   if [ ! -f $TESTDB ]; then
      exit 2
   fi
   
   getScannerID $imss_work_dir
   
   $TESTDB 1 IMSS "$DBUSER" "/P $DBUSERPASS" "$DBNAME"
   if [ $? -ne 0 ]; then
      exit 1
   fi
   
   $TESTDB 103 IMSS "$DBUSER" "/P $DBUSERPASS" "$DBNAME" $NEWIPADDR $SCANNERID
   
   if [ $? -eq 0 ]; then 
      # restart all imssmgr to apply the new IP changes.
      $EUQOP restartmgr  2>/dev/null
      exit 0
   else
      exit 1
   fi 
}

usage()
{
   echo "$0 chkdb master_ip dbuser dbpass"
   echo "$0 reg master_ip dbuser dbpass"
   echo "$0 dereg"
   echo "$0 dereg_without_rescue"
   echo "$0 chng_ip new_ip"
   echo "$0 chng_mastr new_mastr_ip"
   exit 255
}

case "$1" in
   'chkdb')
      # changed by tonylow 1201
      #CheckAdminDB "$2" "$3" "$4"
      #remove_bkconf $imss_work_dir
      check_admin_db_use_tmp_odbc "$2" "$3" "$4"
      if [ "$DBEXIST" = "YES" ]; then
         exit 0
      else
         exit 1
      fi
      ;;
   'reg')
      register "$2" "$3" "$4" "$5" "$6"
      ;;
   'dereg')
      deregister_rescue "$2"
      ;;
   'dereg_without_rescue')
      deregister "$2"
      ;;
   'rescue')
      rescue_with_patch
      ;;
   'chng_ip')
      change_ip "$2"
      ;;
   'chng_mastr')
      change_master "$2"
      ;;
   'getdb')
      get_db_svr_ip
      ;;
   'getchildver')
      get_child_osapp_ver
      ;;
   'getparentver')
      get_parent_osapp_ver "$2" "$3" "$4"
      ;;
   'glcfg')
      ggb_cfg "$2" "$3" "$4" "$5"
      ;;
   'apply_network')
      apply_network
      ;;
   'restart_imss')
      restart_imss
      ;;
   'restart_imss_noui')
      restart_imss_noui
      ;;
   *)
      usage
      ;;
esac
exit 254

