#!/bin/sh

#This script was mapped from WSagent to fulfill a wrapper of ibe_server daemon.
#This daemon serves for all ibe_client instances (embedded in scan daemons) on the same VA box.

PATH=/sbin:/usr/sbin:/bin:/usr/bin:$PATH
export PATH

PS_CMD="ps -ef --width 1000"

IMSS_HOME=/opt/trend/imss
export IMSS_HOME

LD_LIBRARY_PATH=$IMSS_HOME/lib
export LD_LIBRARY_PATH

ODBCINI=$IMSS_HOME/config/odbc.ini
export ODBCINI

ODBCSYSINI=$IMSS_HOME/config/
export ODBCSYSINI

BIN_NAME=ibes
IBES=$IMSS_HOME/bin/ibes
IBES_NAME='Encryption Master Service'
PID_FILE=$IMSS_HOME/bin/ibes.pid

if [ -f /etc/trend-release ]; then
	trap "" SIGINT
	trap "unlock" EXIT

	. $IMSS_HOME/script/imssfunctions

	lock
fi

test_ibes_status()
{
    for i in `$PS_CMD | grep $IBES\$ | grep -v grep | grep -v "ibes -v" | grep -v defunct | awk '{ print $2 }'`
    do
        if test "$i" ; then
            #ibes is running
            return 0 
        fi
    done
    #ibes is not running"
    return 1 
}

start()
{
    for i in `$PS_CMD | grep -w $BIN_NAME | grep -v grep | grep -v defunct|grep -v $BIN_NAME. | awk '{ print $2 }'`
    do 
        if test "$i"
        then 
            echo "$IBES is running."
            return 1
        fi
    done
    if test -x $IBES ; then
        CHECK_IMSSD="$?"
        $IBES &
        sleep 1
        for i in `$PS_CMD | grep -w $BIN_NAME | grep -v grep |grep -v $BIN_NAME. | awk '{ print $2 }'`
        do 
            if test "$i"
            then 
                echo "$IBES has started."
                return 0
            fi
        done 
        echo "$IBES cannot be started."
    else
        echo "$IBES is not executable."
        return 1
    fi
}

stop()
{
    if [ -f $PID_FILE ]; then
        PID=`cat $PID_FILE`
        IMSS_EXIST=`$PS_CMD | grep $PID | grep -w $BIN_NAME | grep -v grep |grep -v $BIN_NAME. `
        if test "$IMSS_EXIST"
        then
            echo "Shutting down $BIN_NAME $PID ..."
            
            kill -INT $PID
            
            COUNT=0
            while $PS_CMD | grep -w $BIN_NAME | grep -v grep|grep -v $BIN_NAME. | grep -v defunct > /dev/null 2>&1 ; do
                sleep 1
                COUNT=`expr $COUNT + 1`
                if test "$COUNT" = "30"
                then
                    for i in `$PS_CMD | grep -w $BIN_NAME | grep -v grep |grep -v $BIN_NAME.|  awk '{ print $2 }'`
                    do
                        echo "Force to kill $i ..."
                        kill -KILL $i
                    done
# case TT293514,don't break out here, reset count to 0, and check the process again
#                    break
                    COUNT=0
                fi
            done  
        else
            echo "$BIN_NAME $PID is not running."
        fi
        rm -f $PID_FILE
    else
        echo "$PID_FILE missing"
        for i in `$PS_CMD |grep -w $BIN_NAME|grep -v grep|grep -v $BIN_NAME. |awk '{print $2}'`
        do
        	kill -KILL $i
        done
    fi
}

#when imssd is runing, prevent dec/enc fail, need to stop imssd first
restart()
{
    test_ibes_status
    ibes_status=$?
    
    imss_status=`$IMSS_HOME/script/S99IMSS status|grep 'is running'`
    
    if [ "$imss_status" != '' -a $ibes_status -eq 0 ]; then
        $IMSS_HOME/script/S99IMSS stop
    fi
    
    if [ $ibes_status -eq 0 ]; then
        stop
    fi
    
    start
    
    if [ "$imss_status" != '' -a $ibes_status -eq 0 ]; then
        $IMSS_HOME/script/S99IMSS start
    fi
}

case "$1" in
'status')
    if ( test_ibes_status ) ; then
	    echo "$IBES is running."
	    exit 1 
    else 
    	echo "$IBES is NOT running."
    	exit 0
    fi
    ;;
'start')
    start
    ;;
'stop')
    stop
    ;;
'restart')
    restart
    ;;              
*)
    echo "usage: $0 start|stop|restart|status"
    exit 1
    ;;
esac
