#!/bin/bash
#
# IMSSCTL		This shell script takes care of starting and stopping
#		        the entire IMSVA Application.
#
# chkconfig: 345 99 10
# description: imssctl is the controller of IMSVA App. \
# vim:shiftwidth=4:tabstop=4:expandtab

IMSS_HOME=/opt/trend/imss

export PATH=/sbin:/usr/sbin:/bin:/usr/bin:$PATH:$IMSS_HOME/postfix/usr/sbin/:$IMSS_HOME/postfix/usr/bin/:$IMSS_HOME/postfix/usr/libexec/postfix

DB=$IMSS_HOME/script/dbctl.sh
POSTFIX=$IMSS_HOME/script/postfix
ADMINUI=$IMSS_HOME/script/S99ADMINUI
CMAGENT=$IMSS_HOME/script/S99CMAGENT

POLICY=$IMSS_HOME/script/S99POLICY
IMSS=$IMSS_HOME/script/S99IMSS
FOXDNS=$IMSS_HOME/script/S99FOXDNS
BIND=$IMSS_HOME/bind/bindctl.sh
FOXPROXY=/opt/trend/ipprofiler/script/foxproxyd
MONITOR=$IMSS_HOME/script/S99MONITOR
SCHEDULED=$IMSS_HOME/script/S99SCHEDULED
MANAGER=$IMSS_HOME/script/S99MANAGER
DTASAGENT=$IMSS_HOME/script/S99DTASAGENT
MSGACTION=$IMSS_HOME/script/msgaction.sh
RTSTAT=$IMSS_HOME/script/rtstat.sh
MAILTRAFFIC=$IMSS_HOME/script/mailtraffic.sh
APACHE=$IMSS_HOME/script/Apache.sh
TOMCAT=$IMSS_HOME/script/S99EUQ
UPDATE=$IMSS_HOME/script/S99UPDATE
MSGTRACING=$IMSS_HOME/script/S99MSGTRACING
OPENLDAP=$IMSS_HOME/script/openldap.sh
IBES=$IMSS_HOME/script/ibe_server.sh

EUQ=$IMSS_HOME/script/S99EUQ
# for generate timezone script
GENTZD=$IMSS_HOME/script/gentzd.sh
TZDFILE=$IMSS_HOME/UI/adminUI/ROOT/javascripts/tzd.js
# For SEG TT 229064, ntpd is not stopped by this script.
NTPD=/etc/init.d/rcNTP.sh

PS_CMD="ps -ef --width 1000"

if [ -f /etc/trend-release ]; then
	trap "" SIGINT
	trap "unlock" EXIT

	. $IMSS_HOME/script/imssfunctions
	. /etc/init.d/functions

	lock
fi

start()
{
    echo "Starting IMSVA Application..."
    if [ -f /etc/trend-release ]; then
		$IBES start&
        $MONITOR start
        $MSGTRACING start
        $OPENLDAP start
        sh $GENTZD > $TZDFILE
        chown imss $TZDFILE
        chgrp imss $TZDFILE
        chmod 750 $TZDFILE
        if [ -d /var/queue_temp/ ] ; then
            /opt/trend/imss/bin/close_handle_sh "nohup rm -rf /var/queue_temp/ > /dev/null 2>&1 &"
        fi
        $DTASAGENT start
    else
    $DB start
    start_others
    fi
    touch /var/lock/subsys/rcImss
    echo "IMSVA Application has been started."
    echo_success
    echo
}
stop_others()
{
    $MONITOR stop
    $MANAGER stop
    $MSGACTION stop
    $RTSTAT stop
    $MAILTRAFFIC stop
    $CMAGENT stop
    $MSGTRACING stop
    $APACHE stop
    $TOMCAT stop
    $FOXPROXY stop
    $POSTFIX stop
    $IMSS stop
    $BIND stop
    $FOXDNS stop
    $POLICY stop
    $SCHEDULED stop
    $UPDATE stop
    $OPENLDAP stop
    $IBES stop
    $DTASAGENT stop
}
stop_others_hang()
{
    $MANAGER stop
    $MSGACTION stop
    $RTSTAT stop
    $MAILTRAFFIC stop
    $CMAGENT stop
    $MSGTRACING stop
    $FOXPROXY stop
    $IMSS stop
    $FOXDNS stop
    $POLICY stop
    $SCHEDULED stop
    $UPDATE stop
    $DTASAGENT stop
}

#[SEGTT-247440] Child IMSVA went down, Parent IMSVA cannot receive mail.
#restart these tasks,because they may hang on db.
restart_others_hang()
{
    #MANAGER will be restarted in monmain.cpp.
    #$MANAGER restart
    $MSGACTION restart
    $RTSTAT restart
    $MAILTRAFFIC restart
    #some user may not use cmagent,so firstly we have to test whether have to restart cmagent
    CMAGENT_ID=`$PS_CMD | grep imsscmagent | grep -v grep | awk '{ print $2 }'`
    if test "$CMAGENT_ID";then
        echo "imsscmagent is running,restart it."
        $CMAGENT restart
    fi
    $MSGTRACING restart
    FOXPROX_ID=`$PS_CMD | grep "ipprofiler/bin/TmFoxProxy" | grep -v grep | awk '{ print $2 }'`
    if test "$FOXPROX_ID";then
        echo "TmFoxProxy is running,restart it."
        $FOXPROXY restart
    fi
    
    $IMSS restart
    $FOXDNS restart
    $POLICY restart
    $SCHEDULED stop
    $SCHEDULED start
    $UPDATE restart
    #some user may not use EUQ,so firstly we have to test whether have to restart EUQ
    EUQ_ID=`$PS_CMD | grep "euqUI\/conf\/server.xml start" | grep -v grep | awk '{ print $2 }'`
    if test "$EUQ_ID";then
        echo "euq is running,restart it."
        $EUQ restart
    fi
    
    #some user may not use DTAS,so firstly we have to test whether have to restart DTAS
    DTAS_ID=`$PS_CMD | grep "bin/dtasagent" | grep -v grep | awk '{ print $2 }'`
    if test "$DTAS_ID";then
        echo "DTAS is running,restart it."
        $DTASAGENT restart
    fi
    
}
stop()
{
    echo "Stopping IMSVA Application..."
    if [ -f /etc/trend-release ]; then
        stop_others
        $ADMINUI stop
        # For SEG TT 291659 merge from 229064, try to stop ntpd.
        # No need to explicitly start ntpd because manager will try to start ntpd when manager itself is started. 
        $NTPD stop
        $DB stop
    else
    	$MONITOR stop
    	$EUQ stop
    	$POLICY stop
    	$IMSS stop
    	$BIND stop
    	$FOXDNS stop
    	$SCHEDULED stop
    	$ADMINUI stop
    	$CMAGENT stop
    	$MSGTRACING stop
    	$POSTFIX stop
    	$DB stop
        $OPENLDAP stop
        $IBES stop
        $DTASAGENT stop
    fi
    rm -f /var/lock/subsys/rcImss
    echo "IMSVA Application has been stopped"
    echo_success
    echo
}
restart_others()
{
    sleep 5
    stop_others
    $DB reload
    start
}

start_others()
{
    $POSTFIX start
    $CMAGENT start
    $ADMINUI start
    $SCHEDULED start
    $FOXDNS start
    $BIND start
    $IMSS start
    $POLICY start
    $EUQ start
    $MSGTRACING start
    $MONITOR start
    $OPENLDAP start
    $IBES start&
}

case "$1" in
    start)
        start
        ;;
    stop)
        stop
        ;;
    stop_others)
        stop_others
        ;;
    stop_others_hang)
        stop_others_hang
        ;;
    start-others)
        start_others
        ;;
    restart)
        stop
        start
        # Fix for case 272508
        service crond restart
        ;;
    restart_others)
    	restart_others
    	;;
    restart_others_hang)
        restart_others_hang
        ;;
    status)
        status imssmgrmon
        status imssmgr
        status imssd
        status imssps
        exit $?
        ;;
    *)
        echo $0 "start|stop|stop_others|restart|restart_others|restart_others_hang|status"
esac
